/// <reference types="node" />
/// <reference types="node" />
import type { ReadStream } from 'fs';
import type { ListRemotesResponse, PushResponse, UpsertRemoteResponse } from './types';
declare class RemotesApiClient {
    private readonly domain;
    private readonly apiKey;
    constructor(domain: string, apiKey: string);
    private getParsedResponse;
    getDefaultBranch(organizationId: string, projectId: string): Promise<string>;
    upsert(organizationId: string, projectId: string, remote: {
        mountPath: string;
        mountBranchName: string;
    }): Promise<UpsertRemoteResponse>;
    push(organizationId: string, projectId: string, payload: PushPayload, files: {
        path: string;
        stream: ReadStream | Buffer;
    }[]): Promise<PushResponse>;
    getRemotesList(organizationId: string, projectId: string, mountPath: string): Promise<ListRemotesResponse>;
    getPush({ organizationId, projectId, pushId, }: {
        organizationId: string;
        projectId: string;
        pushId: string;
    }): Promise<PushResponse>;
}
export declare class ReuniteApiClient {
    domain: string;
    private readonly apiKey;
    remotes: RemotesApiClient;
    constructor(domain: string, apiKey: string);
}
export type PushPayload = {
    remoteId: string;
    commit: {
        message: string;
        branchName: string;
        sha?: string;
        url?: string;
        createdAt?: string;
        namespace?: string;
        repository?: string;
        author: {
            name: string;
            email: string;
            image?: string;
        };
    };
    isMainBranch?: boolean;
};
export {};
